@@ -34,7 +34,6 @@ enum class CityFlags {
 
 
 class City : IsPartOfGameInfoSerialization {
-    @Suppress("JoinDeclarationAndAssignment")
     @Transient
     lateinit var civ: Civilization
 
@@ -210,68 +209,100 @@ class City : IsPartOfGameInfoSerialization {
     fun containsBuildingUnique(uniqueType: UniqueType) =
         cityConstructions.builtBuildingUniqueMap.getUniques(uniqueType).any()
 
-    fun getGreatPersonPercentageBonus(): Int{
-        var allGppPercentageBonus = 0
+    fun getGreatPersonPercentageBonus() = getGreatPersonPercentageBonusBreakdown().sumOf { it.second }
+
+    @Suppress("RemoveExplicitTypeArguments")  // Clearer readability
+    /** Collects percentage boni applying to all GPP. Each returned entry is a Pair naming the source and the integer percentage. */
+    private fun getGreatPersonPercentageBonusBreakdown() = sequence<Pair<String, Int>> {
         for (unique in getMatchingUniques(UniqueType.GreatPersonPointPercentage)) {
             if (!matchesFilter(unique.params[1])) continue
-            allGppPercentageBonus += unique.params[0].toInt()
+            yield((unique.sourceObjectName ?: "Bonus") to unique.params[0].toInt())
         }
 
         // Sweden UP
         for (otherCiv in civ.getKnownCivs()) {
             if (!civ.getDiplomacyManager(otherCiv).hasFlag(DiplomacyFlags.DeclarationOfFriendship))
                 continue
-
-            for (ourUnique in civ.getMatchingUniques(UniqueType.GreatPersonBoostWithFriendship))
-                allGppPercentageBonus += ourUnique.params[0].toInt()
-            for (theirUnique in otherCiv.getMatchingUniques(UniqueType.GreatPersonBoostWithFriendship))
-                allGppPercentageBonus += theirUnique.params[0].toInt()
+            val boostUniques = civ.getMatchingUniques(UniqueType.GreatPersonBoostWithFriendship) +
+                otherCiv.getMatchingUniques(UniqueType.GreatPersonBoostWithFriendship)
+            for (unique in boostUniques)
+                yield("Declaration of Friendship" to unique.params[0].toInt())
         }
-        return allGppPercentageBonus
     }
 
-    fun getGreatPersonPointsForNextTurn(): HashMap<String, Counter<String>> {
-        val sourceToGPP = HashMap<String, Counter<String>>()
+    data class GreatPersonPointsBreakdownEntry(val source: String, val isBonus: Boolean, val counter: Counter<String> = Counter())
+    fun getGreatPersonPointsBreakdown(): Sequence<GreatPersonPointsBreakdownEntry> {
+        // First part: Base GPP points, materialized since we'll need to scan them twice
+        val basePoints = mutableListOf<GreatPersonPointsBreakdownEntry>()
 
-        val specialistsCounter = Counter<String>()
+        // ... from Specialists
+        val specialists = GreatPersonPointsBreakdownEntry("Specialists", false)
         for ((specialistName, amount) in population.getNewSpecialists())
             if (getRuleset().specialists.containsKey(specialistName)) { // To solve problems in total remake mods
                 val specialist = getRuleset().specialists[specialistName]!!
-                specialistsCounter.add(specialist.greatPersonPoints.times(amount))
+                specialists.counter.add(specialist.greatPersonPoints.times(amount))
             }
-        sourceToGPP["Specialists"] = specialistsCounter
+        basePoints.add(specialists)
 
-        val buildingsCounter = Counter<String>()
-        for (building in cityConstructions.getBuiltBuildings())
-            buildingsCounter.add(building.greatPersonPoints)
-        sourceToGPP["Buildings"] = buildingsCounter
+        // ... and from buildings, allowing multiples of the same name to be aggregated since builtBuildingObjects is a List
+        basePoints.addAll(
+            cityConstructions.getBuiltBuildings()
+                .groupBy({ it.name }, { it.greatPersonPoints })
+                .map { GreatPersonPointsBreakdownEntry(
+                    it.key,
+                    false,
+                    it.value.fold(null) { a: Counter<String>?, b -> if (a == null) b else a + b }!! // Just a sumOf<Counter>, !! because groupBy doesn't output empty lists
+                ) }
+        )
 
-        val stateForConditionals = StateForConditionals(civInfo = civ, city = this)
-        for ((_, gppCounter) in sourceToGPP) {
+        return sequence {
+            // Second part: passing the first part through first, but collecting all names with base points while doing so
+            val allGpp = mutableSetOf<String>()
+            for (element in basePoints) {
+                yield(element)
+                allGpp += element.counter.keys
+            }
+
+            // Now add boni: GreatPersonPointPercentage and GreatPersonBoostWithFriendship
+            for ((source, bonus) in getGreatPersonPercentageBonusBreakdown()) {
+                val bonusEntry = GreatPersonPointsBreakdownEntry(source, true)
+                for (gppName in allGpp)
+                    bonusEntry.counter[gppName] = bonus
+                yield(bonusEntry)
+            }
+
+            // And last, the GPP-type-specific GreatPersonEarnedFaster Unique
+            val stateForConditionals = StateForConditionals(civInfo = civ, city = this@City)
             for (unique in civ.getMatchingUniques(UniqueType.GreatPersonEarnedFaster, stateForConditionals)) {
-                val unitName = unique.params[0]
-                if (!gppCounter.containsKey(unitName)) continue
-                gppCounter.add(unitName, gppCounter[unitName] * unique.params[1].toInt() / 100)
+                val bonusEntry = GreatPersonPointsBreakdownEntry(unique.sourceObjectName ?: "Bonus", true)
+                bonusEntry.counter.add(unique.params[0], unique.params[1].toInt())
+                yield(bonusEntry)
             }
-
-            val allGppPercentageBonus = getGreatPersonPercentageBonus()
-
-            for (unitName in gppCounter.keys)
-                gppCounter.add(unitName, gppCounter[unitName] * allGppPercentageBonus / 100)
         }
-
-        return sourceToGPP
     }
 
-    fun getGreatPersonPoints(): Counter<String> {
-        val gppCounter = Counter<String>()
-        for (entry in getGreatPersonPointsForNextTurn().values)
-            gppCounter.add(entry)
+    fun getGreatPersonPoints() = Counter<String>().apply {
+        // Using fixed-point(n.3) math to avoid surprises by rounding while still leveraging the Counter class
+        // Also accumulating boni separately - to ensure they operate additively not multiplicatively
+        val boni = Counter<String>()
+        for ((_, isBonus, counter) in getGreatPersonPointsBreakdown()) {
+            if (!isBonus) {
+                add(counter * 1000)
+            } else {
+                boni.add(counter)
+            }
+        }
+        for (key in keys.filter { it in boni }) {
+            add(key, this[key] * boni[key] / 100)
+        }
+        // round fixed-point to integers
+        for (key in keys)
+            this[key] = (this[key] * 0.001).roundToInt()
+
         // Remove all "gpp" values that are not valid units
-        for (key in gppCounter.keys.toSet())
+        for (key in keys.toSet())
             if (key !in getRuleset().units)
-                gppCounter.remove(key)
-        return gppCounter
+                remove(key)
     }
 
     fun addStat(stat: Stat, amount: Int) {
