@@ -170,7 +170,7 @@ class UnitTurnManager(val unit: MapUnit) {
             UncivGame.Current.settings.addCompletedTutorialTask("Construct an improvement")
 
         val improvementInProgress = tile.improvementInProgress ?: return
-        tile.changeImprovement(improvementInProgress, unit.civ)
+        tile.changeImprovement(improvementInProgress, unit.civ, unit)
 
         tile.improvementInProgress = null
         tile.getCity()?.updateCitizens = true
