@@ -91,6 +91,8 @@ class GameSettings {
     var androidHideSystemUi = true
 
     var multiplayer = GameSettingsMultiplayer()
+    
+    var autoPlay = GameSettingsAutoPlay()
 
     var enableEspionageOption = false
 
@@ -246,9 +248,10 @@ class GameSettings {
 
     enum class NationPickerListMode { Icons, List }
 
-    enum class LocaleCode(var language: String, var country: String) {
+    enum class LocaleCode(val language: String, val country: String, val trueLanguage: String? = null) {
         Arabic("ar", "IQ"),
         Belarusian("be", "BY"),
+        Bosnian("bs", "BA"),
         BrazilianPortuguese("pt", "BR"),
         Bulgarian("bg", "BG"),
         Catalan("ca", "ES"),
@@ -268,6 +271,7 @@ class GameSettings {
         Italian("it", "IT"),
         Japanese("ja", "JP"),
         Korean("ko", "KR"),
+        Latin("la", "IT"),
         Latvian("lv", "LV"),
         Lithuanian("lt", "LT"),
         Malay("ms", "MY"),
@@ -279,6 +283,7 @@ class GameSettings {
         Portuguese("pt", "PT"),
         Romanian("ro", "RO"),
         Russian("ru", "RU"),
+        Rusyn("uk", "UA", "rus"), // No specific locale for rus exists, so use closest for collator
         Serbian("sr", "RS"),
         SimplifiedChinese("zh", "CN"),
         Slovak("sk", "SK"),
@@ -319,6 +324,35 @@ class GameSettings {
         }
     }
 
+    class GameSettingsAutoPlay {
+        var showAutoPlayButton: Boolean = false
+        var autoPlayMaxTurns = 10
+        var fullAutoPlayAI: Boolean = true
+        var autoPlayMilitary: Boolean = true
+        var autoPlayCivilian: Boolean = true
+        var autoPlayEconomy: Boolean = true
+        var autoPlayTechnology: Boolean = true
+        var autoPlayPolicies: Boolean = true
+        var autoPlayReligion: Boolean = true
+        var autoPlayDiplomacy: Boolean = true
+    
+        var turnsToAutoPlay: Int = 0
+        var autoPlayTurnInProgress: Boolean = false
+    
+        fun startAutoPlay() {
+            turnsToAutoPlay = autoPlayMaxTurns
+        }
+        
+        fun stopAutoPlay() {
+            turnsToAutoPlay = 0
+            autoPlayTurnInProgress = false
+        }
+    
+        fun isAutoPlaying(): Boolean = turnsToAutoPlay > 0
+        
+        fun isAutoPlayingAndFullAI():Boolean = isAutoPlaying() && fullAutoPlayAI
+    }
+
     @Suppress("SuspiciousCallableReferenceInLambda")  // By @Azzurite, safe as long as that warning below is followed
     enum class GameSetting(
         val kClass: KClass<*>,
