@@ -3,14 +3,18 @@ package com.unciv.models.ruleset.unique
 import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
 import com.unciv.UncivGame
+import com.unciv.logic.battle.MapUnitCombatant
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.CivFlags
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.LocationAction
 import com.unciv.logic.civilization.MapUnitAction
 import com.unciv.logic.civilization.MayaLongCountAction
+import com.unciv.logic.civilization.NotificationAction
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
+import com.unciv.logic.civilization.PolicyAction
+import com.unciv.logic.civilization.TechAction
 import com.unciv.logic.civilization.managers.ReligionState
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
@@ -20,8 +24,8 @@ import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
 import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.hasPlaceholderParameters
-import com.unciv.ui.components.extensions.addToMapOfSets
 import com.unciv.ui.components.MayaCalendar
+import com.unciv.ui.components.extensions.addToMapOfSets
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsUpgrade
 import kotlin.math.roundToInt
 import kotlin.random.Random
@@ -54,23 +58,27 @@ object UniqueTriggerActivation {
         when (unique.type) {
             UniqueType.OneTimeFreeUnit -> {
                 val unitName = unique.params[0]
-                val unit = ruleSet.units[unitName]
-                if ((chosenCity == null && tile == null)
-                        || unit == null
-                        || unit.isCityFounder() && civInfo.isOneCityChallenger())
+                val baseUnit = ruleSet.units[unitName] ?: return false
+                val unit = civInfo.getEquivalentUnit(baseUnit)
+                if (unit.isCityFounder() && civInfo.isOneCityChallenger())
                     return false
 
                 val limit = unit.getMatchingUniques(UniqueType.MaxNumberBuildable)
                     .map { it.params[0].toInt() }.minOrNull()
-                if (limit != null && limit <= civInfo.units.getCivUnits().count { it.name == unitName })
+                if (limit != null && limit <= civInfo.units.getCivUnits().count { it.name == unit.name })
                     return false
 
-                val placedUnit = if (city != null || tile == null)
-                    civInfo.units.addUnit(unitName, chosenCity) ?: return false
-                    else civInfo.units.placeUnitNearTile(tile.position, unitName) ?: return false
-
+                // 4 situations: If city ->
+                val placedUnit = when {
+                    city != null || (tile == null && civInfo.cities.isNotEmpty()) ->
+                        civInfo.units.addUnit(unit, chosenCity) ?: return false
+                    tile != null -> civInfo.units.placeUnitNearTile(tile.position, unit) ?: return false
+                    civInfo.units.getCivUnits().any() ->
+                        civInfo.units.placeUnitNearTile(civInfo.units.getCivUnits().first().currentTile.position, unit) ?: return false
+                    else -> return false
+                }
                 val notificationText = getNotificationText(notification, triggerNotificationText,
-                    "Gained [1] [$unitName] unit(s)")
+                    "Gained [1] [${unit.name}] unit(s)")
                     ?: return true
 
                 civInfo.addNotification(
@@ -83,13 +91,16 @@ object UniqueTriggerActivation {
             }
             UniqueType.OneTimeAmountFreeUnits -> {
                 val unitName = unique.params[1]
-                val unit = ruleSet.units[unitName]
-                if ((chosenCity == null && tile == null) || unit == null || (unit.isCityFounder() && civInfo.isOneCityChallenger()))
+                val baseUnit = ruleSet.units[unitName]
+                if ((chosenCity == null && tile == null) || baseUnit == null)
+                    return false
+                val unit = civInfo.getEquivalentUnit(baseUnit)
+                if (unit.isCityFounder() && civInfo.isOneCityChallenger())
                     return false
 
                 val limit = unit.getMatchingUniques(UniqueType.MaxNumberBuildable)
                     .map { it.params[0].toInt() }.minOrNull()
-                val unitCount = civInfo.units.getCivUnits().count { it.name == unitName }
+                val unitCount = civInfo.units.getCivUnits().count { it.name == unit.name }
                 val amountFromTriggerable = unique.params[0].toInt()
                 val actualAmount = when {
                     limit == null -> amountFromTriggerable
@@ -101,22 +112,22 @@ object UniqueTriggerActivation {
 
                 val tilesUnitsWerePlacedOn: MutableList<Vector2> = mutableListOf()
                 repeat(actualAmount) {
-                    val placedUnit = if (city != null || tile == null) civInfo.units.addUnit(unitName, chosenCity)
-                        else civInfo.units.placeUnitNearTile(tile.position, unitName)
+                    val placedUnit = if (city != null || tile == null) civInfo.units.addUnit(unit, chosenCity)
+                        else civInfo.units.placeUnitNearTile(tile.position, unit)
                     if (placedUnit != null)
                         tilesUnitsWerePlacedOn.add(placedUnit.getTile().position)
                 }
                 if (tilesUnitsWerePlacedOn.isEmpty()) return true
 
                 val notificationText = getNotificationText(notification, triggerNotificationText,
-                    "Gained [${tilesUnitsWerePlacedOn.size}] [$unitName] unit(s)")
+                    "Gained [${tilesUnitsWerePlacedOn.size}] [${unit.name}] unit(s)")
                     ?: return true
 
                 civInfo.addNotification(
                     notificationText,
                     MapUnitAction(tilesUnitsWerePlacedOn),
                     NotificationCategory.Units,
-                    civInfo.getEquivalentUnit(unit).name
+                    unit.name
                 )
                 return true
             }
@@ -189,7 +200,7 @@ object UniqueTriggerActivation {
                     "You gain the [$policyName] Policy")
                     ?: return true
 
-                civInfo.addNotification(notificationText, NotificationCategory.General, NotificationIcon.Culture)
+                civInfo.addNotification(notificationText, PolicyAction(policyName), NotificationCategory.General, NotificationIcon.Culture)
                 return true
             }
             UniqueType.OneTimeEnterGoldenAge, UniqueType.OneTimeEnterGoldenAgeTurns -> {
@@ -209,7 +220,7 @@ object UniqueTriggerActivation {
                 val greatPeople = civInfo.greatPeople.getGreatPeople()
                 if (unique.type == UniqueType.MayanGainGreatPerson && civInfo.greatPeople.longCountGPPool.isEmpty())
                     civInfo.greatPeople.longCountGPPool = greatPeople.map { it.name }.toHashSet()
-                if (civInfo.isHuman()) {
+                if (civInfo.isHuman() && !UncivGame.Current.settings.autoPlay.isAutoPlayingAndFullAI()) {
                     civInfo.greatPeople.freeGreatPeople++
                     // Anyone an idea for a good icon?
                     if (unique.type == UniqueType.MayanGainGreatPerson) {
@@ -311,7 +322,11 @@ object UniqueTriggerActivation {
                             notification.fillPlaceholders(*(techsToResearch.map { it.name }
                                 .toTypedArray()))
                         else notification
-                    civInfo.addNotification(notificationText, LocationAction(tile?.position),
+                    // Notification click for first tech only, supporting multiple adds little value.
+                    // Relies on RulesetValidator catching <= 0!
+                    val notificationActions: Sequence<NotificationAction> =
+                        LocationAction(tile?.position) + TechAction(techsToResearch.first().name)
+                    civInfo.addNotification(notificationText, notificationActions,
                         NotificationCategory.General, NotificationIcon.Science)
                 }
 
@@ -326,7 +341,7 @@ object UniqueTriggerActivation {
                     "You have discovered the secrets of [$techName]")
                     ?: return true
 
-                civInfo.addNotification(notificationText, NotificationCategory.General, NotificationIcon.Science)
+                civInfo.addNotification(notificationText, TechAction(techName), NotificationCategory.General, NotificationIcon.Science)
                 return true
             }
 
@@ -756,6 +771,12 @@ object UniqueTriggerActivation {
                     unit.civ.addNotification(notification, unit.getTile().position, NotificationCategory.Units) // Do we have a heal icon?
                 return true
             }
+            UniqueType.OneTimeUnitDamage -> {
+                MapUnitCombatant(unit).takeDamage(unique.params[0].toInt())
+                if (notification != null)
+                    unit.civ.addNotification(notification, unit.getTile().position, NotificationCategory.Units) // Do we have a heal icon?
+                return true
+            }
             UniqueType.OneTimeUnitGainXP -> {
                 if (!unit.baseUnit.isMilitary()) return false
                 unit.promotions.XP += unique.params[0].toInt()
