@@ -8,6 +8,7 @@ import com.unciv.json.fromJsonFile
 import com.unciv.json.json
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.metadata.BaseRuleset
+import com.unciv.models.ruleset.BeliefType
 import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
@@ -178,6 +179,7 @@ class RulesetValidator(val ruleset: Ruleset) {
         tryFixUnknownUniques: Boolean
     ) {
         for (reward in ruleset.ruinRewards.values) {
+            if (reward.weight < 0) lines += "${reward.name} has a negative weight, which is not allowed!"
             for (difficulty in reward.excludedDifficulties)
                 if (!ruleset.difficulties.containsKey(difficulty))
                     lines += "${reward.name} references difficulty ${difficulty}, which does not exist!"
@@ -229,6 +231,8 @@ class RulesetValidator(val ruleset: Ruleset) {
         tryFixUnknownUniques: Boolean
     ) {
         for (belief in ruleset.beliefs.values) {
+            if (belief.type == BeliefType.Any || belief.type == BeliefType.None)
+                lines += "${belief.name} type is {belief.type}, which is not allowed!"
             uniqueValidator.checkUniques(belief, lines, true, tryFixUnknownUniques)
         }
     }
@@ -433,8 +437,9 @@ class RulesetValidator(val ruleset: Ruleset) {
         for (building in ruleset.buildings.values) {
             addBuildingErrorRulesetInvariant(building, lines)
 
-            if (building.requiredTech != null && !ruleset.technologies.containsKey(building.requiredTech!!))
-                lines += "${building.name} requires tech ${building.requiredTech} which does not exist!"
+            for (requiredTech: String in building.requiredTechs())
+                if (!ruleset.technologies.containsKey(requiredTech))
+                    lines += "${building.name} requires tech ${requiredTech} which does not exist!"
             for (specialistName in building.specialistSlots.keys)
                 if (!ruleset.specialists.containsKey(specialistName))
                     lines += "${building.name} provides specialist $specialistName which does not exist!"
@@ -575,7 +580,7 @@ class RulesetValidator(val ruleset: Ruleset) {
     }
 
     private fun addBuildingErrorRulesetInvariant(building: Building, lines: RulesetErrorList) {
-        if (building.requiredTech == null && building.cost == -1 && !building.hasUnique(
+        if (building.requiredTechs().none() && building.cost == -1 && !building.hasUnique(
                 UniqueType.Unbuildable
             )
         )
@@ -653,26 +658,30 @@ class RulesetValidator(val ruleset: Ruleset) {
 
     /** Collects all RulesetSpecific checks for a BaseUnit */
     private fun checkUnitRulesetSpecific(unit: BaseUnit, lines: RulesetErrorList) {
-        if (unit.requiredTech != null && !ruleset.technologies.containsKey(unit.requiredTech!!))
-            lines += "${unit.name} requires tech ${unit.requiredTech} which does not exist!"
-        if (unit.obsoleteTech != null && !ruleset.technologies.containsKey(unit.obsoleteTech!!))
-            lines += "${unit.name} obsoletes at tech ${unit.obsoleteTech} which does not exist!"
+        for (requiredTech: String in unit.requiredTechs())
+            if (!ruleset.technologies.containsKey(requiredTech))
+                lines += "${unit.name} requires tech ${requiredTech} which does not exist!"
+        for (obsoleteTech: String in unit.techsAtWhichNoLongerAvailable())
+            if (!ruleset.technologies.containsKey(obsoleteTech))
+                lines += "${unit.name} obsoletes at tech ${obsoleteTech} which does not exist!"
         if (unit.upgradesTo != null && !ruleset.units.containsKey(unit.upgradesTo!!))
             lines += "${unit.name} upgrades to unit ${unit.upgradesTo} which does not exist!"
 
         // Check that we don't obsolete ourselves before we can upgrade
-        if (unit.upgradesTo!=null && ruleset.units.containsKey(unit.upgradesTo!!)
-            && unit.obsoleteTech!=null && ruleset.technologies.containsKey(unit.obsoleteTech!!)) {
-            val upgradedUnit = ruleset.units[unit.upgradesTo!!]!!
-            if (upgradedUnit.requiredTech != null && upgradedUnit.requiredTech != unit.obsoleteTech
-                && !getPrereqTree(unit.obsoleteTech!!).contains(upgradedUnit.requiredTech)
-            )
-                lines.add(
-                    "${unit.name} obsoletes at tech ${unit.obsoleteTech}," +
-                        " and therefore ${upgradedUnit.requiredTech} for its upgrade ${upgradedUnit.name} may not yet be researched!",
-                    RulesetErrorSeverity.Warning
-                )
-        }
+        for (obsoleteTech: String in unit.techsThatObsoleteThis())
+            if (unit.upgradesTo!=null && ruleset.units.containsKey(unit.upgradesTo!!)
+                && ruleset.technologies.containsKey(obsoleteTech)) {
+                val upgradedUnit = ruleset.units[unit.upgradesTo!!]!!
+                for (requiredTech: String in upgradedUnit.requiredTechs())
+                    if (requiredTech != obsoleteTech
+                        && !getPrereqTree(obsoleteTech).contains(requiredTech)
+                    )
+                        lines.add(
+                            "${unit.name} obsoletes at tech ${obsoleteTech}," +
+                                " and therefore ${requiredTech} for its upgrade ${upgradedUnit.name} may not yet be researched!",
+                            RulesetErrorSeverity.Warning
+                        )
+            }
 
         for (resource in unit.getResourceRequirementsPerTurn(StateForConditionals.IgnoreConditionals).keys)
             if (!ruleset.tileResources.containsKey(resource))
