@@ -3,8 +3,8 @@ package com.unciv.models.stats
 import com.badlogic.gdx.graphics.Color
 import com.unciv.logic.civilization.NotificationIcon
 import com.unciv.models.UncivSound
-import com.unciv.ui.components.Fonts
 import com.unciv.ui.components.extensions.colorFromHex
+import com.unciv.ui.components.fonts.Fonts
 
 enum class Stat(
     val notificationIcon: String,
