@@ -139,6 +139,9 @@ class MapParametersTable(
             MapType.archipelago,
             MapType.innerSea,
             MapType.perlin,
+            MapType.fractal,
+            MapType.lakes,
+            MapType.smallContinents,
             if (forMapEditor && mapGeneratedMainType != MapGeneratedMainType.randomGenerated) MapType.empty else null
         )
 
