@@ -1,5 +1,7 @@
 package com.unciv.ui.screens.worldscreen.status
 
+import com.unciv.GUI
+import com.unciv.logic.civilization.managers.TurnManager
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
 import com.unciv.ui.components.extensions.isEnabled
@@ -16,7 +18,6 @@ class NextTurnButton(
     private val worldScreen: WorldScreen
 ) : IconTextButton("", null, 30) {
     private var nextTurnAction = NextTurnAction.Default
-
     init {
 //         label.setFontSize(30)
         labelCell.pad(10f)
@@ -30,10 +31,20 @@ class NextTurnButton(
     fun update() {
         nextTurnAction = getNextTurnAction(worldScreen)
         updateButton(nextTurnAction)
-
-        isEnabled = !worldScreen.hasOpenPopups() && worldScreen.isPlayersTurn
-                && !worldScreen.waitingForAutosave && !worldScreen.isNextTurnUpdateRunning()
-
+        val settings = GUI.getSettings()
+        if (!settings.autoPlay.autoPlayTurnInProgress && settings.autoPlay.isAutoPlaying() 
+            && worldScreen.isPlayersTurn && !worldScreen.waitingForAutosave && !worldScreen.isNextTurnUpdateRunning()) {
+            settings.autoPlay.autoPlayTurnInProgress = true
+            if (!worldScreen.viewingCiv.isSpectator())
+                TurnManager(worldScreen.viewingCiv).automateTurn()
+            worldScreen.nextTurn()
+            settings.autoPlay.turnsToAutoPlay--
+            settings.autoPlay.autoPlayTurnInProgress = false
+        }
+                
+        isEnabled = nextTurnAction.getText (worldScreen) == "AutoPlay" 
+            || (!worldScreen.hasOpenPopups() && worldScreen.isPlayersTurn
+                && !worldScreen.waitingForAutosave && !worldScreen.isNextTurnUpdateRunning())
         if (isEnabled) addTooltip(KeyboardBinding.NextTurn) else addTooltip("")
     }
 
