@@ -14,15 +14,17 @@ import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toStringSigned
 import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.widgets.ScalingTableWrapper
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.overviewscreen.EmpireOverviewCategories
 import com.unciv.ui.screens.victoryscreen.VictoryScreen
 
-internal class WorldScreenTopBarResources(topbar: WorldScreenTopBar) : Table() {
+internal class WorldScreenTopBarResources(topbar: WorldScreenTopBar) : ScalingTableWrapper() {
     private val turnsLabel = "Turns: 0/400".toLabel()
     private data class ResourceActors(val resource: TileResource, val label: Label, val icon: Group)
     private val resourceActors = ArrayList<ResourceActors>(12)
     private val resourcesWrapper = Table()
+    val worldScreen = topbar.worldScreen
 
     // Note: For a proper functioning of the "shift floating buttons down when cramped" feature, it is
     // important that this entire Widget has only the bare minimum padding to its left and right.
@@ -46,7 +48,6 @@ internal class WorldScreenTopBarResources(topbar: WorldScreenTopBar) : Table() {
         resourcesWrapper.defaults().space(defaultPad)
         resourcesWrapper.touchable = Touchable.enabled
 
-        val worldScreen = topbar.worldScreen
         turnsLabel.onClick {
             if (worldScreen.selectedCiv.isLongCountDisplay()) {
                 val gameInfo = worldScreen.selectedCiv.gameInfo
@@ -77,6 +78,8 @@ internal class WorldScreenTopBarResources(topbar: WorldScreenTopBar) : Table() {
     }
 
     fun update(civInfo: Civilization) {
+        resetScale()
+
         val yearText = YearTextUtil.toYearText(
             civInfo.gameInfo.getYear(), civInfo.isLongCountDisplay()
         )
@@ -108,6 +111,6 @@ internal class WorldScreenTopBarResources(topbar: WorldScreenTopBar) : Table() {
             resourcesWrapper.add(label).padTop(resourceAmountDescentTweak)  // digits don't have descenders, so push them down a little
         }
 
-        pack()
+        scaleTo(worldScreen.stage.width)
     }
 }
