@@ -1,11 +1,11 @@
 package com.unciv.app.desktop
 
+import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.backends.lwjgl3.Lwjgl3ApplicationConfiguration
 import com.unciv.UncivGame
 
 class DesktopGame(config: Lwjgl3ApplicationConfiguration) : UncivGame() {
 
-    private val audio = HardenGdxAudio()
     private var discordUpdater = DiscordUpdater()
     private val windowListener = UncivWindowListener()
 
@@ -34,7 +34,7 @@ class DesktopGame(config: Lwjgl3ApplicationConfiguration) : UncivGame() {
     }
 
     override fun installAudioHooks() {
-        audio.installHooks(
+        (Gdx.app as HardenGdxAudio).installHooks(
             musicController.getAudioLoopCallback(),
             musicController.getAudioExceptionHandler()
         )
